@@ -2,42 +2,29 @@
 #
 #         PySceneDetect: Python-Based Video Scene Detector
 #   ---------------------------------------------------------------
-#     [  Site: http://www.bcastell.com/projects/PySceneDetect/   ]
+#     [  Site:   http://www.scenedetect.scenedetect.com/         ]
+#     [  Docs:   http://manual.scenedetect.scenedetect.com/      ]
 #     [  Github: https://github.com/Breakthrough/PySceneDetect/  ]
-#     [  Documentation: http://pyscenedetect.readthedocs.org/    ]
 #
-# Copyright (C) 2014-2021 Brandon Castellano <http://www.bcastell.com>.
-#
-# PySceneDetect is licensed under the BSD 3-Clause License; see the included
-# LICENSE file, or visit one of the following pages for details:
-#  - https://github.com/Breakthrough/PySceneDetect/
-#  - http://www.bcastell.com/projects/PySceneDetect/
-#
-# This software uses Numpy, OpenCV, click, tqdm, simpletable, and pytest.
-# See the included LICENSE files or one of the above URLs for more information.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
-# AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
-# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
-# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+# Copyright (C) 2014-2022 Brandon Castellano <http://www.bcastell.com>.
+# PySceneDetect is licensed under the BSD 3-Clause License; see the
+# included LICENSE file, or visit one of the above pages for details.
 #
+""":py:class:`AdaptiveDetector` compares the difference in content between adjacent frames
+similar to `ContentDetector` except the threshold isn't fixed, but is a rolling average of
+adjacent frame changes. This can help mitigate false detections in situations such as fast
+camera motions.
 
-""" Module: ``scenedetect.detectors.adaptive_detector``
-
-This module implements the :py:class:`AdaptiveDetector`, which compares the
-difference in content between adjacent frames similar to `ContentDetector` except the
-threshold isn't fixed, but is a rolling average of adjacent frame changes. This can
-help mitigate false detections in situations such as fast camera motions.
-
-This detector is available from the command-line interface by using the
-`detect-adaptive` command.
+This detector is available from the command-line as the `detect-adaptive` command.
 """
 
+from logging import getLogger
+
 # PySceneDetect Library Imports
 from scenedetect.detectors import ContentDetector
 
+logger = getLogger('pyscenedetect')
+
 
 class AdaptiveDetector(ContentDetector):
     """Detects cuts using HSV changes similar to ContentDetector, but with a
@@ -47,11 +34,31 @@ class AdaptiveDetector(ContentDetector):
 
     ADAPTIVE_RATIO_KEY_TEMPLATE = "adaptive_ratio{luma_only} (w={window_width})"
 
-    def __init__(self, video_manager, adaptive_threshold=3.0,
-                 luma_only=False, min_scene_len=15, min_delta_hsv=15.0, window_width=2):
-        super(AdaptiveDetector, self).__init__()
-        self.video_manager = video_manager
-        self.min_scene_len = min_scene_len  # minimum length of any given scene, in frames (int) or FrameTimecode
+    def __init__(self,
+                 adaptive_threshold=3.0,
+                 luma_only=False,
+                 min_scene_len=15,
+                 min_delta_hsv=15.0,
+                 window_width=2,
+                 video_manager=None):
+        """
+        Arguments:
+            adaptive_threshold: Threshold value (float) that the calculated frame score must exceed to
+                trigger a new scene (see frame metric adaptive_ratio in stats file).
+            luma_only: Only consider luma/brightness channel (useful for greyscale videos).
+            min_scene_len: Minimum length of any scene.
+            min_delta_hsv: Minimum threshold (float) that the content_val must exceed in order to register as a new'
+                scene. This is calculated the same way that `detect-content` calculates frame score.
+            window_width: Size of window (number of frames) before and after each frame to average together in'
+                order to detect deviations from the mean.
+            video_manager: [DEPRECATED] DO NOT USE. For backwards compatibility only.
+        """
+        # TODO: Remove `video_manager`.
+        if video_manager is not None:
+            logger.error('video_manager is deprecated, use video instead.')
+
+        super().__init__()
+        self.min_scene_len = min_scene_len                                              # minimum length of any given scene, in frames (int) or FrameTimecode
         self.adaptive_threshold = adaptive_threshold
         self.min_delta_hsv = min_delta_hsv
         self.window_width = window_width
@@ -59,11 +66,13 @@ class AdaptiveDetector(ContentDetector):
         self._adaptive_ratio_key = AdaptiveDetector.ADAPTIVE_RATIO_KEY_TEMPLATE.format(
             window_width=window_width, luma_only='' if not luma_only else '_lum')
 
+        self._first_frame = None
+        self._last_frame = None
 
     def get_metrics(self):
         # type: () -> List[str]
         """ Combines base ContentDetector metric keys with the AdaptiveDetector one. """
-        return super(AdaptiveDetector, self).get_metrics() + [self._adaptive_ratio_key]
+        return super().get_metrics() + [self._adaptive_ratio_key]
 
     def stats_manager_required(self):
         # type: () -> bool
@@ -93,21 +102,21 @@ class AdaptiveDetector(ContentDetector):
         # Call the process_frame function of ContentDetector but ignore any
         # returned cuts
         if self.is_processing_required(frame_num):
-            super(AdaptiveDetector, self).process_frame(
-                frame_num=frame_num, frame_img=frame_img)
+            super().process_frame(frame_num=frame_num, frame_img=frame_img)
+
+        if self._first_frame is None:
+            self._first_frame = frame_num
+        self._last_frame = frame_num
 
         return []
 
-
     def get_content_val(self, frame_num):
         """
         Returns the average content change for a frame.
         """
-        metric_key = (ContentDetector.FRAME_SCORE_KEY if not self._luma_only
-            else ContentDetector.DELTA_V_KEY)
-        return self.stats_manager.get_metrics(
-            frame_num, [metric_key])[0]
-
+        metric_key = (
+            ContentDetector.FRAME_SCORE_KEY if not self._luma_only else ContentDetector.DELTA_V_KEY)
+        return self.stats_manager.get_metrics(frame_num, [metric_key])[0]
 
     def post_process(self, _):
         """
@@ -120,9 +129,8 @@ class AdaptiveDetector(ContentDetector):
         more than a single frame.
         """
         cut_list = []
-        _, start_timecode, end_timecode = self.video_manager.get_duration()
-        start_frame = start_timecode.get_frames()
-        end_frame = end_timecode.get_frames()
+        if self._first_frame is None:
+            return []
         adaptive_threshold = self.adaptive_threshold
         window_width = self.window_width
         last_cut = None
@@ -131,7 +139,8 @@ class AdaptiveDetector(ContentDetector):
 
         if self.stats_manager is not None:
             # Loop through the stats, building the adaptive_ratio metric
-            for frame_num in range(start_frame + window_width + 1, end_frame - window_width):
+            for frame_num in range(self._first_frame + window_width + 1,
+                                   self._last_frame - window_width):
                 # If the content-val of the frame is more than
                 # adaptive_threshold times the mean content_val of the
                 # frames around it, then we mark it as a cut.
@@ -155,17 +164,18 @@ class AdaptiveDetector(ContentDetector):
                     # is still very low
                     adaptive_ratio = 0.0
 
-                self.stats_manager.set_metrics(
-                    frame_num, {self._adaptive_ratio_key: adaptive_ratio})
+                self.stats_manager.set_metrics(frame_num,
+                                               {self._adaptive_ratio_key: adaptive_ratio})
 
             # Loop through the frames again now that adaptive_ratio has been calculated to detect
             # cuts using adaptive_ratio
-            for frame_num in range(start_frame + window_width + 1, end_frame - window_width):
+            for frame_num in range(self._first_frame + window_width + 1,
+                                   self._last_frame - window_width):
                 # Check to see if adaptive_ratio exceeds the adaptive_threshold as well as there
                 # being a large enough content_val to trigger a cut
                 if (self.stats_manager.get_metrics(
-                    frame_num, [self._adaptive_ratio_key])[0] >= adaptive_threshold and
-                        self.get_content_val(frame_num) >= self.min_delta_hsv):
+                        frame_num, [self._adaptive_ratio_key])[0] >= adaptive_threshold
+                        and self.get_content_val(frame_num) >= self.min_delta_hsv):
 
                     if last_cut is None:
                         # No previously detected cuts
@@ -179,4 +189,4 @@ class AdaptiveDetector(ContentDetector):
             return cut_list
 
         # Stats manager must be used for this detector
-        return None
+        return []
